import Foundation
import RealmSwift

let securityAppGroup: String = "group.snikket.engine"

struct Database {
    let db: Realm

    init() {
        do {
            var dbConfig = Realm.Configuration.init()
            dbConfig.fileURL = Database.storeURL
            dbConfig.schemaVersion = Database.actualSchemaVersion
            dbConfig.migrationBlock = Database.migrations
            dbConfig.encryptionKey = Database.encryptionKey
            db = try Realm(configuration: dbConfig)
        } catch {
            fatalError("Realm DB initialization")
        }

        #if DEBUG
            print("DB path: \(Database.storeURL)")
            print("DB key: \(Database.encryptionKey)")
        #endif
    }
}

extension Database {
    static var storeURL: URL {
        guard let containerURL = FileManager.default.containerURL(forSecurityApplicationGroupIdentifier: securityAppGroup) else {
            fatalError("No group url for database. Make sure that app group set correctly")
        }
        return containerURL.appendingPathComponent("main.realm")
    }
}
