import Foundation
import SwiftUI
import Engine

struct AccountsAddScreen: View {
    @EnvironmentObject var store: AppStore

    private enum Field {
        case jid
        case password
    }
    @FocusState private var focus: Field?

    @State private var jid: String = ""
    @State private var pass: String = ""

    var body: some View {
        VStack {
            Spacer()
            HStack {
                VStack(alignment: .leading) {
                    Text(L10n.Accounts.Add.Exist.title)
                        .font(.head1)
                        .foregroundColor(.Main.black)
                        .onTapGesture {
                            focus = nil
                        }
                    Text(L10n.Accounts.Add.Exist.Prompt.jid)
                        .font(.body1)
                        .foregroundColor(.Main.gray)
                        .padding(.top)
                    UniversalInputCollection.TextField(
                        prompt: L10n.Accounts.Add.Exist.Hint.jid,
                        text: $jid,
                        focus: $focus,
                        fieldType: .jid,
                        contentType: .emailAddress,
                        keyboardType: .emailAddress,
                        submitLabel: .next,
                        action: {
                            focus = .password
                        })

                    Text(L10n.Accounts.Add.Exist.Prompt.password)
                        .font(.body2)
                        .foregroundColor(.Main.gray)
                    UniversalInputCollection.SecureField(
                        prompt: L10n.Accounts.Add.Exist.Hint.password,
                        text: $pass,
                        focus: $focus,
                        fieldType: .password,
                        submitLabel: .done,
                        action: {
                            focus = nil
                        })
                }
            }
                .padding()
            VStack(spacing: 8) {
                Text(L10n.Accounts.Add.or)
                    .font(.body1)
                    .foregroundColor(.Main.gray)
                Button {
                } label: {
                    Text(L10n.Accounts.Add.Exist.Btn.link)
                        .font(.body1)
                        .foregroundColor(.Tango.orangeMedium)
                }
            }

            Spacer()
            Button {
                focus = nil
            } label: {
                Text(L10n.Accounts.Add.Exist.Btn.main)
            }
                .buttonStyle(PrimaryButtonStyle())
                .padding(.horizontal, 80)
                .padding(.bottom, 60)
        }
        .ignoresSafeArea()
        .background(Color.Main.backgroundLight)
            .onTapGesture {
                focus = nil
            }
    }
}

#if DEBUG
    struct AccountsAddScreen_Previews: PreviewProvider {
        static var previews: some View {
            showPreview(stateConfig: { state in
                state.currentFlow = .accounts
                state.accountsState.navigation = .addAccount(.new)
            })
        }
    }
#endif
