import Foundation
import Combine

public typealias ModuleName = String

public protocol ModuleIdentifiable {
    var moduleName: ModuleName { get }
}

public typealias ModuleAction = ModuleIdentifiable & Codable

public typealias ModuleState = Any & ModuleIdentifiable & Codable & Equatable

public typealias GlobalState = [ModuleName: any ModuleState]

public typealias OptionalState = (any ModuleState)?

public protocol Module: ModuleIdentifiable {
    func getInitState() -> OptionalState

    func reduce(state: inout any ModuleState, action: any ModuleAction)

    func middleware(state: GlobalState, action: any ModuleAction) -> AnyPublisher<any ModuleAction, Never>
}

// Some modules reacts on outside events, this variable related to Store dispatch (some kind of callback)
protocol ModuleContinuousActing {
    var asyncDispatch: ((_ action: any ModuleAction) -> Void)? { get set }
}
