import UIKit
import TigaseSwift

// FIXME: solve linter suppressed issues
// swiftlint: disable force_unwrapping
class AvatarStatusView: UIView {
    @IBOutlet var avatarImageView: AvatarView!
    @IBOutlet var statusImageView: UIImageView! {
        didSet {
            statusImageView.backgroundColor = self.backgroundColor
        }
    }

    override var backgroundColor: UIColor? {
        get {
            return super.backgroundColor
        }
        set {
            super.backgroundColor = newValue
            statusImageView?.backgroundColor = newValue
        }
    }

    override func awakeFromNib() {
        super.awakeFromNib()
        // avatarImageView.image = UIImage(named: "first")
        updateCornerRadius()
    }

    func set(bareJID: BareJID? = nil, name: String?, avatar: UIImage?, orDefault defAvatar: UIImage) {
        self.avatarImageView.set(bareJID: bareJID, name: name, avatar: avatar, orDefault: defAvatar)
    }

    func setGroup(bareJIDS: [BareJID], memberNames: [String], memberImages: [UIImage], groupAvatar: UIImage?, defAvatar: UIImage) {
        self.avatarImageView.setGroup(bareJIDS: bareJIDS, memberNames: memberNames, memberImages: memberImages, avatar: groupAvatar, defAvatar: defAvatar)
    }

    func setStatus(_ status: Presence.Show?) {
        let image: UIImage? = AvatarStatusView.getStatusImage(status)

        statusImageView.image = image
    }

    static func getStatusImage(_ status: Presence.Show?) -> UIImage? {
        // default color as for offline contact
        var image: UIImage? = UIImage(named: "presence_offline")
        if let status {
            switch status {
            case .chat:
                image = UIImage(named: "presence_chat")

            case .online:
                image = UIImage(named: "presence_online")

            case .away:
                image = UIImage(named: "presence_away")

            case .xa:
                image = UIImage(named: "presence_xa")

            case .dnd:
                image = UIImage(named: "presence_dnd")
            }
        }
        return image
    }

    static func drawStatusBorder(backgroundColor: UIColor, status: UIImage) -> UIImage {
        let scale = UIScreen.main.scale
        let size = status.size

        UIGraphicsBeginImageContextWithOptions(size, false, scale)
        print("size:", size, "scale:", scale)
        let ctx = UIGraphicsGetCurrentContext()!

        let path = CGPath(ellipseIn: CGRect(origin: .zero, size: size), transform: nil)
        ctx.addPath(path)

        ctx.setFillColor(backgroundColor.cgColor)
        ctx.fill(CGRect(x: 0, y: 0, width: size.width, height: size.height))

        status.draw(in: CGRect(x: 0, y: 0, width: size.width, height: size.height))

        let image = UIGraphicsGetImageFromCurrentImageContext()!
        UIGraphicsEndImageContext()

        return image
    }

    func drawStatusIcon(_ size: CGFloat, color: UIColor) -> UIImage {
        UIGraphicsBeginImageContextWithOptions(CGSize(width: size, height: size), false, 0)
        let ctx = UIGraphicsGetCurrentContext()
        ctx!.saveGState()

        let rect = CGRect(x: 0, y: 0, width: size, height: size)
        ctx!.setFillColor(color.cgColor)
        ctx!.fillEllipse(in: rect)

        ctx!.restoreGState()
        let img = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()

        return img!
    }

    override func layoutSubviews() {
        super.layoutSubviews()
        updateCornerRadius()
    }

    func updateCornerRadius() {
        avatarImageView.layer.masksToBounds = true
        avatarImageView.layer.cornerRadius = 3
        statusImageView.layer.opacity = 1.0
        statusImageView.layer.masksToBounds = true
        statusImageView.layer.cornerRadius = self.statusImageView.frame.height / 2
    }
}
